

/*
           panteltje huaweic.

         Copyright (C) Jan Panteltje 2008

 email: panteltje@yahoo.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "huaweic.h"

int script_flag;


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc( strlen(s) +  1);
if(! p) return 0;

strcpy(p, s);
return p;
} /* end function strsave */


int send_data(int fd, char *buffer, int length)
{
int b;
char text[2048];

if(debug_flag)
	{
	fprintf(stderr, "send_data(): arg fd=%d buffer=%p length=%d\n", fd, buffer, length);

	strncpy(text, buffer, length);
	text[length] = 0;
	fprintf(stderr, "text=%s\n\r", text);
	}

if(length == 0) return 1;

//ssize_t write(int fd, const void *buf, size_t count);
b = write(fd, buffer, length);
if(b == -1)
	{
	fprintf(stderr, "huaweic: write to device %s failed because ", device);
	perror("");				

	fprintf(stderr, "aborting.\n");
			
	if(! script_flag) system("stty sane");

	exit(1);
	}

if(b != length)
	{
	fprintf(stderr,\
	"huaweic: could only write %d of %d bytes to device %s.\n", b, length, device);
	}

return 1;
} /* end function send_data */


int set_port_parameters(int fd, int baudrate, char *device)
{
struct termios tty;
speed_t baud;

if(debug_flag)
	{
	fprintf(stdout, "set_port_parameters(): arg fd=%d baudrate=%d device=%s\n", fd, baudrate, device);
	}

/* convert real baudrates to 'B' baudrates */
switch(baudrate)
	{
	case 9600:
		baud = B9600;
		break;
	case 19200:
		baud = B19200;
		break;
	case 38400:
		baud = B38400;
		break;
	case 57600:
		baud = B57600;
		break;
	case 115200:
		baud = B115200;
		break;
	default:
		fprintf(stderr, "baudrate=%d unsupported, aborting.\n", baudrate);
		return 0;
		break;
	} /* end switch baudrate */

/* set comport parameters */
tcgetattr(fd, &tty);

tty.c_cflag = 0;
tty.c_cflag |= CLOCAL;			/* ignore modem control lines */
tty.c_cflag |= CREAD;			/* enable receiver */
tty.c_cflag |= CS8;				/* use 8 data bits */
//tty.c_cflag |= CSTOPB;		/* 2 stop bits */
//tty.c_cflag |= PARENB;		/* enable parity */
//tty.c_cflag |= PARODD;		/* use odd parity */
//tty.c_cflag &= ~CRTSCTS;		/* do not use RTS and CTS handshake */

tty.c_iflag = 0;
tty.c_iflag |= IGNBRK;
tty.c_iflag |= IGNPAR;
//tty.c_iflag |= PARMRK
//tty.c_iflag |= INPCK;
//tty.c_iflag |= INLCR;
//tty.c_iflag |= ISTRIP
//tty.c_iflag &= ~(IXON | IXOFF | IXANY);

tty.c_oflag = 0;

tty.c_lflag = 0;
//tty.c_lflag |= ICANON;
//tty.c_cc[VMIN] = 1;
//tty.c_cc[VTIME] = 5;
//tty.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);


cfsetospeed(&tty, (speed_t)baud);
cfsetispeed(&tty, (speed_t)baud);

tcsetattr(fd, TCSANOW, &tty);

return 1;
} /* end function set_port_parameters */


void print_usage()
{
fprintf(stdout, "Panteltje huaweic-%s serial communication program.\n", VERSION);
fprintf(stdout, "Usage: huaweic [-b baudrate] [-d device_name] [-h] [-v]\n");
fprintf(stdout,\
"\
b n        baudrate, either 9600, 19200, 38400, 57600, or 115200, default 19200.\n\
d device   device name, default /dev/ttyUSB0.\n\
i          script flag, use this to send single command from a script,\n\
            example: echo a1 | huaweic -i , or echo a1 | huaweic -i > result.\n\
h          help (this help).\n\
v          verbose, prints functions and arguments.\n\
Escape exits.\n\
\n\
ctrl P to enter PIN\n\
ctrl L to list all SMS messages\n\
ctrl D to delete all SMS messages\n\
");

} /* end function print_usage */



int main(int argc, char **argv)
{
int a, b, i;
int fd;
fd_set readfs; /* data type represents file descriptor for select function, (actually it is a bit array) */
struct timeval timeout;
char rx_buffer[1024];
char tx_buffer[1024];
int device_nr;
int baudrate;
int state = 0;
char pin[5];
int pin_pos = 0; // -Wall

/* defaults */
device_nr = 0;
/* set com parameters 19200 8 bit no parity */
baudrate = 115200;
script_flag = 0;

device = strsave("/dev/ttyUSB0");
if(! device)
	{
	fprintf(stderr, "huaweic: could not allocate space for device name, aborting.\n");
	
	exit(1);
	}

strcpy(pin, "");
/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "b:d:ihv");
	if(a == -1) break;

	switch(a)
		{
		case 'b':
			baudrate = atoi(optarg);
			break;			
		case 'd':
			free(device);
			device = strsave(optarg);
			if(! device)
				{
				fprintf(stderr, "huaweic: could not allocate space for device name, aborting.\n");
	
				exit(1);
				}
			break;
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'i':
			script_flag = 1;
			break;
		case 'v': // debug
			debug_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		} /* end switch a */
	} /* end while getopt() */

if(! device)
	{
	fprintf(stderr, "huaweic: main: could not allocate space for device, aborting.\n");

	exit(1);
	}

fd = open(device, O_RDWR);
if(fd  == -1)
	{
	fprintf(stderr, "huaweic: could not open device %s for read-write, aborting.\n", device);
	
	exit(1);
	}

// int set_port_parameters(int fd, int baudrate, char *device)
if(! set_port_parameters(fd, baudrate, device) )
	{
	fprintf(stderr, "huaweic: could not set baudrate %d, aborting.\n", baudrate); 
	
	exit(1);
	}

if(! script_flag) 
	{
	/* ID */
	fprintf(stdout, "Panteltje huaweic-%s.\n", VERSION);

	/* How to abort */
	fprintf(stdout, "Escape exits.\n\
\n\
ctrl P to enter PIN\n\
ctrl L to list all SMS messages\n\
ctrl D to delete all SMS messages\n\
");

	/* no wait for ENTER */
	system("stty raw");

	/* no echo */
	system("stty -echo");
	} /* end if ! script_flag */

/* communications loop */
while(1)
	{
	/* this macro initializes the file descriptor readfs to to be the empty set */
	FD_ZERO(&readfs);

	/* this macro adds fd to the file descriptor readfs */
	FD_SET(fd, &readfs);

	/* this macro adds fileno(stdin) to the file descriptor readfs */
	FD_SET(fileno(stdin), &readfs);

	timeout.tv_sec = 100;
	timeout.tv_usec = 0;

	/* int FD_SETSIZE  macro is maximum number of filedescriptors that fd_set can hold */
 	/* function select waits for specified filedescr. to have a signal */
	/* last argument struct timeval *timeout */
	b = select(FD_SETSIZE, &readfs, NULL, NULL, &timeout);
	if(b == -1) /* 0 is timeout, -1 error (in errno) */
		{
		fprintf(stderr, "huaweic: select: failed, because ");
		perror("");
		fprintf(stderr, "aborting.\n");

		exit(1);		
		}

	/* test if user has data */
	if( FD_ISSET(fileno(stdin), &readfs) )
		{	
		a = read(fileno(stdin), tx_buffer, 1023);

//fprintf(stderr, "WAS read tx_bufder[0]=%c\n\r", tx_buffer[0]);

		if(state == 2) // PIN
			{
//fprintf(stderr, "WAS tx_buffer=%s pin_pos=%d\n", tx_buffer, pin_pos);
			pin[pin_pos] = tx_buffer[0];
			pin_pos++;
			if(pin_pos == 4)
				{
				pin[pin_pos] = 0;
				sprintf(tx_buffer, "AT+CPIN=\"%s\"%c%c", pin, 13, 0);

				send_data(fd, tx_buffer, strlen(tx_buffer) );

				state = 0;
				}

			a = 0;
			continue;
			}

		for(i = 0; i < a; i++)
			{
			if(tx_buffer[i] == 27) /* Escape */
				{
				if(! script_flag) system("stty sane");
			
				exit(1);
				}

			 if(state == 1) // reply to y/n
				{
				if(tx_buffer[0] == 'y')
					{
					sprintf(tx_buffer, "AT+CMGD=0,4%c%c", 13, 10);
//					sprintf(tx_buffer, "AT+CMFG=0%c%c", 13, 10);

					send_data(fd, tx_buffer, strlen(tx_buffer) );
					}
				else
					{
					fprintf(stderr, "DELETE CANCELLED\r\n");
					}
				state = 0;

				a = 0;
				continue;
				}

			if(tx_buffer[0] == 12) // ctrl L	LIST ALL SMS
				{
				sprintf(tx_buffer, "AT+CMGF=1%c", 13);
				send_data(fd, tx_buffer, strlen(tx_buffer) );

				sprintf(tx_buffer, "AT+CMGL=\"ALL\"%c", 13);
				send_data(fd, tx_buffer, strlen(tx_buffer) );

				a = 0;
				continue;
				}

			if(tx_buffer[0] == 4) // ctrl D		DELETE
				{
				fprintf(stderr, "Delete all SMS(y/n)?\r\n");

				state = 1;	

				a = 0;
				
				continue;
				}

			if(tx_buffer[0] == 16) // ctrl P ENTER PIN
				{
				fprintf(stderr, "Pincode?\r\n");

				pin_pos = 0;

				state = 2;				

				break;
				}

//fprintf(stderr, "tx_buffer=%s a=%d\n", tx_buffer, a);				
				
			/* PIC wants CR */
			if(tx_buffer[i] == 10) tx_buffer[i] = 13;

			} /* end for all available characters */

		if(state == 2) continue;

		send_data(fd, tx_buffer, a);

		} /* end if user has data */

	/* test if remote side has data */
	if( FD_ISSET(fd, &readfs) )
		{
//		ssize_t read(int fd, void *buf, size_t count);
		a = read(fd, rx_buffer, 1023);

		for(i = 0; i < a; i++)
			{
			fprintf(stdout, "%c", rx_buffer[i]);		

			if(script_flag)
				{
				if(rx_buffer[i] == 10) exit(0);
				}
			}
		} /* end for all available characters from remote side */
	
	} /* end while com loop */

exit(0);
} /* end main */



